@@ -1,7 +1,7 @@
 //! Monolithic encoder and decoder.
 use std::io::{self, Read, Write};
 
-use {Decode, DecodeBuf, Encode, EncodeBuf, ErrorKind, Result};
+use {ByteCount, Decode, Encode, Eos, ErrorKind, Result};
 use bytes::BytesEncoder;
 
 /// This trait allows for decoding items monolithically from a source byte stream.
@@ -48,15 +48,18 @@ impl<D: MonolithicDecode> MonolithicDecoder<D> {
 impl<D: MonolithicDecode> Decode for MonolithicDecoder<D> {
     type Item = D::Item;
 
-    fn decode(&mut self, buf: &mut DecodeBuf) -> Result<Option<Self::Item>> {
-        if buf.is_eos() {
-            let item = track!(self.inner.monolithic_decode((&self.buf[..]).chain(buf)))?;
+    fn decode(&mut self, mut buf: &[u8], eos: Eos) -> Result<(usize, Option<Self::Item>)> {
+        if eos.is_reached() {
+            let original_len = buf.len();
+            let item = track!(
+                self.inner
+                    .monolithic_decode(self.buf.as_slice().chain(buf.by_ref()))
+            )?;
             self.buf.clear();
-            Ok(Some(item))
+            Ok((original_len - buf.len(), Some(item)))
         } else {
             self.buf.extend_from_slice(buf);
-            buf.consume_all();
-            Ok(None)
+            Ok((buf.len(), None))
         }
     }
 
@@ -64,8 +67,8 @@ impl<D: MonolithicDecode> Decode for MonolithicDecoder<D> {
         false
     }
 
-    fn is_idle(&self) -> bool {
-        self.buf.is_empty()
+    fn requiring_bytes(&self) -> ByteCount {
+        ByteCount::Unknown
     }
 }
 
@@ -115,7 +118,7 @@ impl<E: MonolithicEncode> MonolithicEncoder<E> {
 impl<E: MonolithicEncode> Encode for MonolithicEncoder<E> {
     type Item = E::Item;
 
-    fn encode(&mut self, mut buf: &mut EncodeBuf) -> Result<()> {
+    fn encode(&mut self, mut buf: &mut [u8], eos: Eos) -> Result<usize> {
         if let Some(item) = self.item.take() {
             let mut extra = Vec::new();
             {
@@ -126,7 +129,7 @@ impl<E: MonolithicEncode> Encode for MonolithicEncoder<E> {
                 track!(self.buf.start_encoding(extra))?;
             }
         }
-        track!(self.buf.encode(buf))
+        track!(self.buf.encode(buf, eos))
     }
 
     fn start_encoding(&mut self, item: Self::Item) -> Result<()> {
@@ -138,6 +141,16 @@ impl<E: MonolithicEncode> Encode for MonolithicEncoder<E> {
     fn is_idle(&self) -> bool {
         self.item.is_none() && self.buf.is_idle()
     }
+
+    fn requiring_bytes(&self) -> ByteCount {
+        if self.is_idle() {
+            ByteCount::Finite(0)
+        } else if self.item.is_some() {
+            ByteCount::Unknown
+        } else {
+            self.buf.requiring_bytes()
+        }
+    }
 }
 
 #[derive(Debug)]
